package api

import (
	"errors"
	"fmt"
	"queencount/db/model"
	"queencount/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func AssetCount(router *GinRouter) {
	// 统计总信息
	router.GET("/count/info/types/:dbId", GetAssetCount)
	// 统计总图表
	router.GET("/count/info", GetInfo)
}

type AssetDefCountInfo struct {
	Mesh     []CountDefItem `json:"mesh"`
	Image    []CountDefItem `json:"image"`
	Material []CountDefItem `json:"material"`
	Env3d    []CountDefItem `json:"env3d"`
}

type CountDefItem struct {
	DefItemNum int    `json:"defItemNum"`
	DefName    string `json:"defName"`
	DefKey     string `json:"defKey"`
}

func GetAssetCount(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// databases := []string{"mesh", "image", "material", "env3d"}
	// 定义模型数量
	dbId := c.Param("dbId")
	if dbId == "" {
		return nil, errors.New("dbId不能为空")
	}
	query := make(map[string]interface{})
	query["_id"] = dbId
	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionDatabase,
		Query:       query,
		Project:     []string{"_id", "assets"},
	}
	var datbaseDoc model.Database
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), option, &datbaseDoc)
	if err != nil || !found {
		return nil, err
	}

	var countDefitem CountDefItem
	var assetDefCountInfo AssetDefCountInfo

	defMeshSet := make([]CountDefItem, 0)
	defImageSet := make([]CountDefItem, 0)
	defMaterialSet := make([]CountDefItem, 0)
	defEnv3dSet := make([]CountDefItem, 0)

	dbName := datbaseDoc.Name

	for _, v := range datbaseDoc.Assets {
		// 模型 mesh
		if v.Type == 10 {
			countDefitem.DefKey = v.Collection
			countDefitem.DefName = v.Label

			parm := &repo.DbDocsSearchOptions{
				Db: dbName,
				// Query:       map[string]interface{}{"ownerId": apictx.User.ID},
				CollectName: v.Collection,
				Project:     []string{"_id"},
			}

			ok, res := repo.DbRepoSeachDocsMap(apictx.CreateRepoCtx(), parm)
			if ok {
				countDefitem.DefItemNum = len(res)
				defMeshSet = append(defMeshSet, countDefitem)
				assetDefCountInfo.Mesh = defMeshSet
			}

		}
		if v.Type == 20 {
			countDefitem.DefKey = v.Collection
			countDefitem.DefName = v.Label

			parm := &repo.DbDocsSearchOptions{
				Db: dbName,
				// Query:       map[string]interface{}{"ownerId": apictx.User.ID},
				CollectName: v.Collection,
				Project:     []string{"_id"},
			}

			ok, res := repo.DbRepoSeachDocsMap(apictx.CreateRepoCtx(), parm)
			if ok {
				countDefitem.DefItemNum = len(res)
				defImageSet = append(defImageSet, countDefitem)
				assetDefCountInfo.Image = defImageSet

			}

		}
		if v.Type == 30 {
			countDefitem.DefKey = v.Collection
			countDefitem.DefName = v.Label

			parm := &repo.DbDocsSearchOptions{
				Db: dbName,
				// Query:       map[string]interface{}{"ownerId": apictx.User.ID},
				CollectName: v.Collection,
				Project:     []string{"_id"},
			}

			ok, res := repo.DbRepoSeachDocsMap(apictx.CreateRepoCtx(), parm)
			if ok {
				countDefitem.DefItemNum = len(res)
				defMaterialSet = append(defMaterialSet, countDefitem)
				assetDefCountInfo.Material = defMaterialSet

			}
		}
		if v.Type == 40 {
			countDefitem.DefKey = v.Collection
			countDefitem.DefName = v.Label

			parm := &repo.DbDocsSearchOptions{
				Db: dbName,
				// Query:       map[string]interface{}{"ownerId": apictx.User.ID},
				CollectName: v.Collection,
				Project:     []string{"_id"},
			}

			ok, res := repo.DbRepoSeachDocsMap(apictx.CreateRepoCtx(), parm)
			if ok {
				countDefitem.DefItemNum = len(res)
				defEnv3dSet = append(defEnv3dSet, countDefitem)
				assetDefCountInfo.Env3d = defEnv3dSet

			}
		}

	}

	return assetDefCountInfo, nil
}

type CountInfo struct {
	Mesh     []DefItem `json:"mesh"`
	Image    []DefItem `json:"image"`
	Material []DefItem `json:"material"`
	Env3d    []DefItem `json:"env3d"`
}

type DefItem struct {
	DefName string      `json:"defName"`
	DefKey  string      `json:"defKey"`
	Time    []CountTime `json:"time"`
}
type CountTime struct {
	TimePoint time.Time `json:"timePoint"`
	Count     int       `json:"count"`
}

type DefInfo struct {
	DefId   string
	DefName string
	DefKey  string
}

type DefCateInfo struct {
	Mesh     []DefInfo
	Image    []DefInfo
	Material []DefInfo
	Env3d    []DefInfo
}

// 查找类型分类信息
func GetDefCate(dbId string, ctx *repo.RepoSession) (defCateInfo DefCateInfo, err error) {
	var database model.Database
	var defInfo DefInfo
	query := make(map[string]interface{})
	query["_id"], _ = primitive.ObjectIDFromHex(dbId)

	param := &repo.DocSearchOptions{
		CollectName: repo.CollectionDatabase,
		Query:       query,
		Project:     []string{"_id", "assets"},
	}
	ok, err := repo.RepoSeachDoc(ctx, param, &database)
	if err != nil || !ok {
		return defCateInfo, errors.New("查询数据错误")
	}

	for _, v := range database.Assets {
		defInfo.DefKey = v.Collection
		defInfo.DefName = v.Label
		defInfo.DefId = v.Id

		// mesh
		if v.Type == 10 {
			defCateInfo.Mesh = append(defCateInfo.Mesh, defInfo)
		}
		// image
		if v.Type == 20 {
			defCateInfo.Image = append(defCateInfo.Image, defInfo)
		}
		// material
		if v.Type == 30 {
			defCateInfo.Material = append(defCateInfo.Material, defInfo)
		}
		// env3d
		if v.Type == 40 {
			defCateInfo.Env3d = append(defCateInfo.Env3d, defInfo)
		}

	}

	return defCateInfo, nil
}

// TODO 需要优化
type ReqAssetCount struct {
	DbId      string `json:"dbId"`
	Step      int    `json:"step"`
	StartTime string `json:"startTime"`
	EndTime   string `json:"endTime"`
}

// 获取统计信息图表需要的信息
func GetInfo(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	var form ReqAssetCount
	err := c.ShouldBindJSON(&form)
	if err != nil {
		return nil, errors.New("参数错误")
	}

	if form.Step == 0 {
		return nil, errors.New("时间步距不能为空")
	}
	if form.StartTime == "" {
		return nil, errors.New("开始时间不能为空")
	}
	if form.EndTime == "" {
		return nil, errors.New("结束时间不能为空")
	}
	if form.DbId == "" {
		return nil, errors.New("数据库id不为空")
	}

	ctx := apictx.CreateRepoCtx()
	timeSet := GetCountTimeSet(form)

	query := make(map[string]interface{})
	// query["dbId"], _ = primitive.ObjectIDFromHex(form.DbId)
	dbId, _ := primitive.ObjectIDFromHex(form.DbId)

	// ??? 需要修改
	// 根据id查询数据库名
	var databaseDoc model.Database
	dbParam := &repo.DocSearchOptions{
		CollectName: repo.CollectionDatabase,
		Query:       map[string]interface{}{"_id": dbId},
		Project:     []string{"_id", "name", "assets"},
	}
	is, err := repo.RepoSeachDoc(ctx, dbParam, &databaseDoc)
	if !is || err != nil {
		return nil, errors.New("资产数据为空")
	}

	// 查数据名下对应的assetcount
	parm := &repo.DocSearchOptions{
		Db:          databaseDoc.Name,
		CollectName: repo.CollectionAssetCount,
		Project:     []string{"_id", "defId", "createTime"},
		Sort:        bson.M{"createTime": -1}, // 降序 小于等于createTime中取时间最近的 第一条
	}

	var assetCount model.AssetCount

	// 循环asset.id ->defId,defKey,defName

	if databaseDoc.Assets == nil {
		return nil, errors.New("资产不存在")
	}
	// 返回的数据
	var countInfo CountInfo
	// 各项数据
	var meshCountTime CountTime
	var meshDefItem DefItem

	var imageCountTime CountTime
	var imageDefItem DefItem

	var materialCountTime CountTime
	var materialDefItem DefItem

	var env3dCountTime CountTime
	var env3dDefItem DefItem

	for _, asset := range databaseDoc.Assets {
		// mesh
		if asset.Type == 10 {
			meshDefItem.DefKey = asset.Collection
			meshDefItem.DefName = asset.Label

			for _, timePoint := range timeSet {
				// 拼接条件 查询各个增量时间段的数据
				query["defId"] = asset.Id
				parm.Query = query
				bl, err := repo.RepoSeachDoc(ctx, parm, &assetCount)
				if !bl || err != nil {
					meshCountTime.Count = 0
				} else {
					meshCountTime.Count = assetCount.Count
				}
				meshCountTime.TimePoint = timePoint
				meshDefItem.Time = append(meshDefItem.Time, meshCountTime)

			}
			countInfo.Mesh = append(countInfo.Mesh, meshDefItem)
		}

		// image

		if asset.Type == 20 {
			imageDefItem.DefKey = asset.Collection
			imageDefItem.DefName = asset.Label

			for _, timePoint := range timeSet {
				// 拼接条件 查询各个增量时间段的数据
				query["defId"] = asset.Id
				parm.Query = query
				bl, err := repo.RepoSeachDoc(ctx, parm, &assetCount)
				if !bl || err != nil {
					imageCountTime.Count = 0
				} else {
					imageCountTime.Count = assetCount.Count
				}
				imageCountTime.TimePoint = timePoint
				imageDefItem.Time = append(imageDefItem.Time, imageCountTime)

			}
			countInfo.Image = append(countInfo.Image, imageDefItem)
		}

		// material
		if asset.Type == 30 {
			materialDefItem.DefKey = asset.Collection
			materialDefItem.DefName = asset.Label

			for _, timePoint := range timeSet {
				// 拼接条件 查询各个增量时间段的数据
				query["defId"] = asset.Id
				parm.Query = query
				bl, err := repo.RepoSeachDoc(ctx, parm, &assetCount)
				if !bl || err != nil {
					imageCountTime.Count = 0
				} else {
					materialCountTime.Count = assetCount.Count
				}
				materialCountTime.TimePoint = timePoint
				materialDefItem.Time = append(materialDefItem.Time, materialCountTime)

			}
			countInfo.Material = append(countInfo.Material, materialDefItem)
		}

		// env3d
		if asset.Type == 40 {
			env3dDefItem.DefKey = asset.Collection
			env3dDefItem.DefName = asset.Label

			for _, timePoint := range timeSet {
				// 拼接条件 查询各个增量时间段的数据
				query["defId"] = asset.Id
				parm.Query = query
				bl, err := repo.RepoSeachDoc(ctx, parm, &assetCount)
				if !bl || err != nil {
					env3dCountTime.Count = 0
				} else {
					env3dCountTime.Count = assetCount.Count
				}
				env3dCountTime.TimePoint = timePoint
				env3dDefItem.Time = append(env3dDefItem.Time, env3dCountTime)

			}
			countInfo.Env3d = append(countInfo.Env3d, env3dDefItem)
		}

	}

	return countInfo, nil
}

// 获取需要查询数据的时间点
func GetCountTimeSet(form ReqAssetCount) (timeSet []time.Time) {
	// 计算有多少个时间段
	startTime, _ := time.Parse("2006-01-02 15:04:05", form.StartTime)
	endTime, _ := time.Parse("2006-01-02 15:04:05", form.EndTime)
	duration := endTime.Unix() - startTime.Unix()
	counts := int(duration / int64(form.Step)) // 取整

	for i := 0; i <= counts; i++ {
		b, _ := time.ParseDuration(fmt.Sprintf("%ds", i*form.Step))
		start := startTime.Add(b)
		// 第一个元素为开始时间
		timeSet = append(timeSet, start)
	}
	return
}
